import { forwardRef, memo, useEffect, useImperativeHandle } from 'react'
import useUserStore from '@/store/userStore'
import { dictMapping, useNoAuthDict } from '@/hooks/useDict'
import { DatePicker, Form, Input, InputNumber, Radio, Select, Switch, Upload } from 'antd'
import { PlusOutlined } from '@ant-design/icons'

// 基础比较条件类型（原子条件）
type CompareCondition = {
  type: 'COMPARE' // COMPARE代表原子条件 仅进行对比，不进行逻辑运算
  operator: 'eq' | 'neq' | 'gt' | 'gte' | 'lt' | 'lte' | 'in' // 扩展其他运算符
  field: string // 支持点分路径如 user.age
  value: any // 可细化类型为 string | number | Array<any> 等
}
// 逻辑运算符类型（可嵌套）
type LogicalCondition = {
  type: 'AND' | 'OR' | 'NOT' | 'NONE'
  conditions: Condition[] // 递归引用
}
// 复合条件类型
type Condition = LogicalCondition | CompareCondition

interface DependenciesItem {
  triggerFields: string[]
  condition: Condition
  action: {
    type: 'show' | 'hide' | 'fetch'
    params?: {
      url?: string
      params?: Record<string, any>
      method?: 'get' | 'post' | 'put' | 'delete' | 'patch' | 'head' | 'options' | 'link' | 'unlink'
      headers?: Record<string, any>
      body?: Record<string, any>
    }
  }
}
type Dependencies = DependenciesItem[]

const { RangePicker } = DatePicker

export interface DynamicFormRef {
  onSubmit: () => void
}

interface DynamicFormProps {
  formItems?: API.TmplItem[]
  formData?: Record<string, any>
  formRule?: any
  disabled?: boolean
  setFormData: (data: Record<string, any>) => void
}

const DynamicForm = memo(
  forwardRef<DynamicFormRef, DynamicFormProps>(
    ({ formItems = [], formData = {}, formRule = {}, disabled = false }, ref) => {
      useImperativeHandle(ref, () => {
        return {
          onSubmit
        }
      })

      const { dictArray: inputUnitTypeOptions, dictRequestFn: getInputUnitTypeOptions } =
        useNoAuthDict('input_unit_type')
      useEffect(() => {
        getInputUnitTypeOptions()
      }, [])

      // form相关
      const [form] = Form.useForm()
      const onSubmit = () => {
        form.submit()
      }
      const onFinish = (values: any) => {
        console.log('Finish:', values)
      }
      const onFinishFailed = (errorInfo: any) => {
        console.log('Failed:', errorInfo)
      }

      // 1.1生成事件处理对象
      const getEventHandlers = (field: API.TmplItem) => {
        const { widgetId, events } = field.itemStruct!
        const handlers: Record<string, (e: any) => any> = {}
        Object.entries(events || {}).forEach(([eventNames, config]) => {
          // eventNames首字母大写
          handlers['on' + eventNames.charAt(0).toUpperCase() + eventNames.slice(1)] = (e) =>
            handlerEvent(e, eventNames, config, widgetId!)
        })
        return handlers
      }
      // 1.2自定义事件处理函数
      const handlerEvent = (value: any, eventNames: string, eventConfig: any, widgetId: string) => {
        console.log('自定义事件处理函数')
        console.log(value, eventNames, eventConfig, widgetId)
        // 这里可以实现事件的统一处理逻辑
        // 例如：表单联动、远程数据加载、字段验证等
        // switch (eventConfig.type) {
        //   case 'linkage':
        //     // 处理表单联动
        //     form.setFieldsValue(eventConfig.targetValues)
        //     break
        //   case 'remote':
        //     // 处理远程数据加载
        //     fetchData(eventConfig.url, value).then((data) => {
        //       form.setFieldsValue(data)
        //     })
        //     break
        //   // 其他事件类型...
        // }
      }

      // 2.1收集依赖
      const dependenciesMap = new Map<string, Map<string, Dependencies>>()
      const registerDependency = () => {
        formItems.forEach((comp) => {
          // { gender: { make: [] } }
          if (comp.itemStruct?.dependencies) {
            comp.itemStruct?.dependencies.forEach((dep: any) => {
              const { triggerFields } = dep
              triggerFields.forEach((field: string) => {
                let fieldDep = dependenciesMap.get(field)
                if (!fieldDep) {
                  dependenciesMap.set(field, (fieldDep = new Map()))
                }
                const depMap = fieldDep.get(comp.itemStruct!.widgetId!) || []
                fieldDep.set(comp.itemStruct!.widgetId!, [...depMap, dep])
              })
            })
          }
        })
      }
      registerDependency()
      console.log(dependenciesMap)
      // 2.2触发依赖
      const compareCondition = (conditions: CompareCondition[], formData: any): boolean => {
        const condition = conditions[0]
        switch (condition.operator) {
          case 'eq':
            return formData[condition.field] === condition.value
          case 'neq':
            return formData[condition.field] !== condition.value
          case 'gt':
            return formData[condition.field] > condition.value
          case 'gte':
            return formData[condition.field] >= condition.value
          default:
            return false
        }
      }
      const evaluateCondition = (condition: Condition, formData: any): boolean => {
        switch (condition.type) {
          case 'AND':
            return (condition.conditions as LogicalCondition[]).every((condition) =>
              evaluateCondition(condition, formData)
            )
          case 'OR':
            return (condition.conditions as LogicalCondition[]).some((condition) =>
              evaluateCondition(condition, formData)
            )
          case 'NOT':
            return !compareCondition(condition.conditions as CompareCondition[], formData)
          case 'NONE':
            return compareCondition(condition.conditions as CompareCondition[], formData)
          case 'COMPARE':
            return compareCondition([condition as CompareCondition], formData)
          default:
            return false
        }
      }
      const triggerDependency = (field: string, _value: any, allValues: any) => {
        const depMap = dependenciesMap.get(field)
        console.log(depMap)
        if (depMap) {
          for (const [key, dep] of depMap) {
            dep.forEach((depItem) => {
              const { condition, action: _action } = depItem
              // condition条件为true才执行action
              console.log(evaluateCondition(condition, allValues))
              if (evaluateCondition(condition, allValues)) {
                console.log(key, '执行action')
                // action.type === 'show' && (allValues[comp.prop] = true)
              }
            })
          }
        }
      }
      // 监听表单数据变动
      const onValuesChange = (changedValues: any, allValues: any) => {
        // console.log(changedValues)
        // console.log(allValues)
        // console.log(formData)
        console.log(allValues)
        triggerDependency(Object.keys(changedValues)[0], Object.values(changedValues)[0], allValues)
      }
      // 表单项包裹
      const FieldWrapper = (props: { children: React.ReactNode }) => {
        return (
          <div className='px-[20px] py-[16px] mb-[16px] rounded-[10px] bg-[#ecf5ff]'>
            {props.children}
          </div>
        )
      }
      // 表单项生成器
      const renderField = (field: API.TmplItem) => {
        const options = field.itemStruct
        const { type, label, desc, placeholder, visible, expand, widgetId } = options!
        if (!visible) return null // 条件渲染

        const labelContent = (
          <div>
            <div>{label.text}</div>
            {desc && (
              <div className='text-text-secondary mt-[4px]' style={{ fontSize: '12px' }}>
                {desc}
              </div>
            )}
          </div>
        )
        switch (type) {
          case 'INPUT': {
            const expandVal = expand as API.FD_COM['INPUT']
            const handlers = getEventHandlers(field)

            return (
              <FieldWrapper key={widgetId}>
                <Form.Item
                  name={widgetId}
                  label={labelContent}
                  rules={formRule[widgetId!]}
                  hasFeedback
                  style={{ marginBottom: 0 }}>
                  <Input
                    placeholder={placeholder}
                    prefix={expandVal.prefix}
                    suffix={expandVal.suffix}
                    showCount
                    allowClear
                    {...handlers}
                  />
                </Form.Item>
              </FieldWrapper>
            )
          }
          case 'NUMBER_INPUT': {
            const expandVal = expand as API.FD_COM['NUMBER_INPUT']
            const min = typeof expandVal.min === 'number' ? expandVal.min : undefined
            const max = typeof expandVal.max === 'number' ? expandVal.max : undefined
            const decimalDigits = expandVal.decimalPlacesNumber || 0

            return (
              <FieldWrapper key={widgetId}>
                <Form.Item name={widgetId} label={labelContent} style={{ marginBottom: 0 }}>
                  <InputNumber
                    className='!w-full'
                    placeholder={`${min !== undefined ? '最小值：' + min.toFixed(decimalDigits) + '；' : ''}${
                      max !== undefined ? '最大值：' + max.toFixed(decimalDigits) + '；' : ''
                    }${'小数位数：' + decimalDigits + '；'}`}
                    maxLength={parseInt(expandVal.maxChars)}
                    prefix={expandVal.prefix}
                    suffix={
                      (expandVal.suffix ||
                        (expandVal.featureType === 'NUMBER' && expandVal.unit)) &&
                      (expandVal.featureType === 'NUMBER' && expandVal.unit !== 'NONE'
                        ? expandVal.unit
                        : expandVal.suffix)
                    }
                    min={min}
                    max={max}
                    precision={decimalDigits}
                  />
                </Form.Item>
              </FieldWrapper>
            )
          }
          case 'AMOUNT_INPUT': {
            const expandVal = expand as API.FD_COM['NUMBER_INPUT']
            const min = typeof expandVal.min === 'number' ? expandVal.min : undefined
            const max = typeof expandVal.max === 'number' ? expandVal.max : undefined
            const decimalDigits = expandVal.decimalPlacesNumber || 0

            return (
              <FieldWrapper key={widgetId}>
                <Form.Item name={widgetId} label={labelContent} style={{ marginBottom: 0 }}>
                  <InputNumber
                    className='!w-full'
                    placeholder={`${min !== undefined ? '最小值：' + min.toFixed(decimalDigits) + '；' : ''}${
                      max !== undefined ? '最大值：' + max.toFixed(decimalDigits) + '；' : ''
                    }${'小数位数：' + decimalDigits + '；'}`}
                    maxLength={parseInt(expandVal.maxChars)}
                    prefix={expandVal.prefix}
                    suffix={
                      (expandVal.suffix || expandVal.unit) &&
                      (expandVal.unit
                        ? dictMapping({ unit: expandVal.unit }, 'unit', inputUnitTypeOptions)
                        : expandVal.suffix)
                    }
                    min={min}
                    max={max}
                    precision={decimalDigits}
                  />
                </Form.Item>
              </FieldWrapper>
            )
          }
          case 'SELECT': {
            const expandVal = expand as API.FD_COM['SELECT']

            return (
              <FieldWrapper key={widgetId}>
                <Form.Item name={widgetId} label={labelContent} style={{ marginBottom: 0 }}>
                  <Select
                    mode={expandVal.multiple ? 'multiple' : undefined}
                    placeholder={placeholder}
                    options={expandVal.userInput}
                    allowClear
                  />
                </Form.Item>
              </FieldWrapper>
            )
          }
          case 'RADIO': {
            const expandVal = expand as API.FD_COM['RADIO']
            const handlers = getEventHandlers(field)

            return (
              <FieldWrapper key={widgetId}>
                <Form.Item name={widgetId} label={labelContent} style={{ marginBottom: 0 }}>
                  <Radio.Group options={expandVal.userInput} {...handlers} />
                </Form.Item>
              </FieldWrapper>
            )
          }
          case 'SWITCH': {
            // const expandVal = expand as API.FD_COM['SWITCH']
            const handlers = getEventHandlers(field)

            return (
              <FieldWrapper key={widgetId}>
                <Form.Item name={widgetId} label={labelContent} style={{ marginBottom: 0 }}>
                  <Switch {...handlers} />
                </Form.Item>
              </FieldWrapper>
            )
          }
          case 'DATETIME_PICKER': {
            const expandVal = expand as API.FD_COM['DATETIME_PICKER']

            const datePickerType = expandVal.dateType.toLowerCase()
            const datePickerTypeMap = {
              year: { picker: 'year', showTime: false, dateFormat: 'YYYY' },
              month: { picker: 'month', showTime: false, dateFormat: 'YYYY-MM' },
              date: { picker: undefined, showTime: false, dateFormat: 'YYYY-MM-DD' },
              week: { picker: 'week', showTime: false, dateFormat: 'YYYY-WW' },
              datetime: { picker: undefined, showTime: true, dateFormat: 'YYYY-MM-DD HH:mm:ss' },
              daterange: { picker: 'range', showTime: false, dateFormat: 'YYYY-MM-DD' },
              datetimerange: { picker: 'range', showTime: true, dateFormat: 'YYYY-MM-DD HH:mm:ss' }
            }
            if (!datePickerType.includes('range')) {
              return (
                <FieldWrapper key={widgetId}>
                  <Form.Item name={widgetId} label={labelContent} style={{ marginBottom: 0 }}>
                    <DatePicker
                      className='w-full'
                      picker={
                        datePickerTypeMap[datePickerType as keyof typeof datePickerTypeMap]
                          .picker as any
                      }
                      showTime={
                        datePickerTypeMap[datePickerType as keyof typeof datePickerTypeMap].showTime
                      }
                      format={
                        datePickerTypeMap[datePickerType as keyof typeof datePickerTypeMap]
                          .dateFormat
                      }
                    />
                  </Form.Item>
                </FieldWrapper>
              )
            } else {
              return (
                <FieldWrapper key={widgetId}>
                  <Form.Item name={widgetId} label={labelContent} style={{ marginBottom: 0 }}>
                    <RangePicker
                      className='w-full'
                      picker={
                        datePickerTypeMap[datePickerType as keyof typeof datePickerTypeMap]
                          .picker as any
                      }
                      showTime={
                        datePickerTypeMap[datePickerType as keyof typeof datePickerTypeMap].showTime
                      }
                      format={
                        datePickerTypeMap[datePickerType as keyof typeof datePickerTypeMap]
                          .dateFormat
                      }
                    />
                  </Form.Item>
                </FieldWrapper>
              )
            }
          }
          case 'FILE_PICKER': {
            const expandVal = expand as API.FD_COM['FILE_PICKER']
            const uploadUrl = '/cms/file/submit'
            const userStore = useUserStore()
            const headers = {
              Authorization: 'Bearer ' + userStore.access_token,
              'X-Tenant-Id': String(userStore.userInfo.tenantId),
              'X-Target-Folder': 'DECLARATION',
              'X-Client-Type': 'TO_B',
              'X-Terminal-Type': ''
            }

            const limitType = (() => {
              const accept = expandVal.accept || ''
              const multiAcceptList = expandVal.multiAcceptList || []
              if (accept && multiAcceptList.length !== 0) {
                if (multiAcceptList[0] === 'ATTACHMENT') {
                  return undefined
                } else {
                  return multiAcceptList.join(',')
                }
              } else if (accept) {
                if (accept === 'ATTACHMENT') {
                  return undefined
                } else {
                  return accept
                }
              } else if (multiAcceptList.length !== 0) {
                if (multiAcceptList[0] === 'ATTACHMENT') {
                  return undefined
                } else {
                  return multiAcceptList.join(',')
                }
              } else {
                return undefined
              }
            })()
            const normFile = (e: any) => {
              // console.log(e)
              if (Array.isArray(e)) return e
              return e?.fileList || []
            }

            // console.log(limitType)

            const handlePreview = (file: any) => {
              console.log(file)
              // window.open(file.url)
            }
            const handleChange = (file: any) => {
              console.log(file)
            }

            const uploadButton = (
              <button style={{ border: 0, background: 'none' }} type='button'>
                <PlusOutlined />
                <div style={{ marginTop: 8 }}>点击上传</div>
              </button>
            )
            return (
              <FieldWrapper key={widgetId}>
                <Form.Item
                  name={widgetId}
                  label={labelContent}
                  style={{ marginBottom: 0 }}
                  valuePropName='fileList'
                  getValueFromEvent={normFile}>
                  <Upload
                    action={import.meta.env.VITE_APP_ENV === 'dev' ? '/api' + uploadUrl : uploadUrl}
                    headers={headers}
                    listType='picture-card'
                    accept={limitType}
                    maxCount={expandVal.limit}
                    onPreview={handlePreview}
                    onChange={handleChange}>
                    {uploadButton}
                  </Upload>
                </Form.Item>
              </FieldWrapper>
            )
          }
          // 其他组件类型...
          default:
            return null
        }
      }

      return (
        <Form
          form={form}
          layout='vertical'
          initialValues={formData}
          onValuesChange={onValuesChange}
          disabled={disabled}
          onFinish={onFinish}
          onFinishFailed={onFinishFailed}>
          {formItems.map(renderField)}
        </Form>
      )
    }
  )
)

export default DynamicForm
